<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2023 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 *  If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Core\Admin;

class EventStream
{
    private string $event;
    private string $data;

    public function __construct($data, $event = '')
    {
        $this->event = $event;
        $this->data = $data;
    }

    public function __toString()
    {
        $stream = '';
        if ($this->event) {
            $stream .= "event: {$this->event}\n";
        }
        $stream .= "data: {$this->data}";
        $stream .= \str_pad('', 4096, "\n");

        return $stream;
    }

    public static function initialize(): void
    {
        \date_default_timezone_set('UTC');
        @\header('X-Accel-Buffering: no');
        @\header('Cache-Control: no-cache');
        @\header('Content-Type: text/event-stream');
        if (\function_exists('apache_setenv')) {
            @\apache_setenv('no-gzip', '1');
        }
        @\ini_set('zlib.output_compression', '0');
        @\ini_set('implicit_flush', '1');
        \ob_implicit_flush();
        \set_time_limit(0);
    }

    public static function send($data, $event = ''): void
    {
        $eventStream = new \JchOptimize\Core\Admin\EventStream($data, $event);
        echo $eventStream;
        while (\ob_get_level() > 0) {
            \ob_end_flush();
        }
        @\ob_flush();
        if (\connection_aborted()) {
            exit;
        }
    }
}
